@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	22 November 1989
- * Last Edited:	31 January 1996
+ * Last Edited:	14 July 1996
  *
  * Copyright 1996 by the University of Washington
  *
@@ -617,7 +617,7 @@ long mail_create (MAILSTREAM *stream,char *mailbox)
 long mail_delete (MAILSTREAM *stream,char *mailbox)
 {
   DRIVER *factory = mail_valid (stream,mailbox,"delete mailbox");
-  return factory ? (*factory->delete) (stream,mailbox) : NIL;
+  return factory ? (*factory->mbxdel) (stream,mailbox) : NIL;
 }
 
 
@@ -637,7 +637,7 @@ long mail_rename (MAILSTREAM *stream,char *old,char *new)
     mm_log (tmp,ERROR);
     return NIL;
   }
-  return factory ? (*factory->rename) (stream,old,new) : NIL;
+  return factory ? (*factory->mbxren) (stream,old,new) : NIL;
 }
 
 /* Mail open
@@ -1153,8 +1153,8 @@ long mail_parse_date (MESSAGECACHE *elt,char *s)
 				/* skip over possible day of week */
   if (isalpha (*s) && isalpha (s[1]) && isalpha (s[2]) && (s[3] == ',') &&
       (s[4] == ' ')) s += 5;
-				/* parse first number (probable month) */
-  if (!(s && (m = strtol ((const char *) s,&s,10)))) return NIL;
+  while (*s == ' ') s++;	/* parse first number (probable month) */
+  if (!(m = strtol ((const char *) s,&s,10))) return NIL;
 
   switch (*s) {			/* different parse based on delimiter */
   case '/':			/* mm/dd/yy format */
@@ -1162,6 +1162,7 @@ long mail_parse_date (MESSAGECACHE *elt,char *s)
 	  (y = strtol ((const char *) ++s,&s,10)) && *s == '\0')) return NIL;
     break;
   case ' ':			/* dd mmm yy format */
+    while (s[1] == ' ') s++;	/* slurp extra whitespace */
   case '-':			/* dd-mmm-yy format */
     d = m;			/* so the number we got is a day */
 				/* make sure string long enough! */
@@ -1182,7 +1183,7 @@ long mail_parse_date (MESSAGECACHE *elt,char *s)
     case (('O'-'A') * 1024) + (('C'-'A') * 32) + ('T'-'A'): m = 10; break;
     case (('N'-'A') * 1024) + (('O'-'A') * 32) + ('V'-'A'): m = 11; break;
     case (('D'-'A') * 1024) + (('E'-'A') * 32) + ('C'-'A'): m = 12; break;
-    default: return NIL;
+    default: return NIL;	/* unknown month */
     }
     if ((s[4] == *s) &&	(y = strtol ((const char *) s+5,&s,10)) &&
 	(*s == '\0' || *s == ' ')) break;
@@ -1217,15 +1218,16 @@ long mail_parse_date (MESSAGECACHE *elt,char *s)
 	elt->zminutes = (s[2] - '0') * 10 + (s[3] - '0');
 	break;
       }
+      while (s[1] == ' ') s++;	/* slurp extra whitespace */
 				/* falls through */
 
     case '-':			/* symbolic time zone */
-      if (!(ms = *++s)) return NIL;
-      if (*++s) {		/* multi-character? */
+      if (!(ms = *++s)) ms = 'Z';
+      else if (*++s) {		/* multi-character? */
 	ms -= 'A'; ms *= 1024;	/* yes, make compressed three-byte form */
 	ms += ((*s++ - 'A') * 32);
 	if (*s) ms += *s++ - 'A';
-	if (*s) return NIL;	/* more than three characters */
+	if (*s) ms = 'Z';	/* more than three characters */
       }
       /* This is not intended to be a comprehensive list of all possible
        * timezone strings.  Such a list would be impractical.  Rather, this
@@ -1325,9 +1327,8 @@ long mail_parse_date (MESSAGECACHE *elt,char *s)
       elt->zminutes = 0;	/* never a fractional hour */
       break;
     case '\0':			/* no time zone */
-      break;
-    default:
-      return NIL;
+    default:			/* bogus time zone */
+      break;			/* ignore both cases */
     }
   }
   else {			/* make sure all time fields zero */
