@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	1 August 1988
- * Last Edited:	9 February 1996
+ * Last Edited:	9 April 1996
  *
  * Copyright 1996 by the University of Washington
  *
@@ -176,6 +176,7 @@ TCPSTREAM *tcp_open (char *host,char *service,long port)
   while ((i = connect (sock,(struct sockaddr *) &sin,sizeof (sin))) < 0 &&
 	 errno == EINTR);
   if (i < 0) switch (errno) {	/* failed? */
+  case EAGAIN:			/* DG brain damage */
   case EINPROGRESS:
   case EISCONN:
   case EADDRINUSE:
