@@ -1,6 +1,6 @@
 #line 2 "os_win.c"		/* So compiler knows original name of this file.*/
 /*
- * $Id: os_win.c,v 4.8 1994/09/27 15:59:37 mikes Exp $
+ * $Id: os_win.c,v 4.34 1996/07/09 23:08:44 mikes Exp $
  *
  * Program:	Operating system dependent routines - MS Windows 3.1
  *
@@ -15,28 +15,16 @@
  *
  * Please address all bugs and comments to "pine-bugs@cac.washington.edu"
  *
- * Copyright 1991-1994  University of Washington
  *
- *  Permission to use, copy, modify, and distribute this software and its
- * documentation for any purpose and without fee to the University of
- * Washington is hereby granted, provided that the above copyright notice
- * appears in all copies and that both the above copyright notice and this
- * permission notice appear in supporting documentation, and that the name
- * of the University of Washington not be used in advertising or publicity
- * pertaining to distribution of the software without specific, written
- * prior permission.  This software is made available "as is", and
- * THE UNIVERSITY OF WASHINGTON DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
- * WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT LIMITATION ALL IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND IN
- * NO EVENT SHALL THE UNIVERSITY OF WASHINGTON BE LIABLE FOR ANY SPECIAL,
- * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, TORT
- * (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, ARISING OUT OF OR IN CONNECTION
- * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * Pine and Pico are trademarks of the University of Washington.
- * No commercial use of these trademarks may be made without prior
- * written permission of the University of Washington.
+ * Pine and Pico are registered trademarks of the University of Washington.
+ * No commercial use of these trademarks may be made without prior written
+ * permission of the University of Washington.
+ * 
+ * Pine, Pico, and Pilot software and its included text are Copyright
+ * 1989-1996 by the University of Washington.
+ * 
+ * The full text of our legal notices is contained in the file called
+ * CPYRIGHT, included with this distribution.
  *
  *
  * Notes:
@@ -55,7 +43,6 @@
 
 #include 	<stdio.h>
 #include	<errno.h>
-#include	<signal.h>
 #include	<setjmp.h>
 #include	<time.h>
 #include	<fcntl.h>
@@ -63,6 +50,7 @@
 /*#include	<bios.h>*/
 
 #include	"osdep.h"
+#include	"pico.h"
 #include	"estruct.h"
 #include	"efunc.h"
 #include        "edef.h"
@@ -78,9 +66,9 @@
 /*
  * Internal functions...
  */
+int timeout = 0;
 static int	MapMSKEYtoPK (int c);
 static int	ProcessMouse(MEvent *, unsigned int *);
-static void	invert_label(int, MENUITEM *);
 int		ttopen ();
 int		ttclose ();
 
@@ -93,6 +81,7 @@ int		ttclose ();
 
 #define	MARGIN			8	/* size of minimim margin and	*/
 #define	SCRSIZ			64	/* scroll size for extended lines */
+#define	MROW			2	/* rows in menu */
 
 
 
@@ -106,6 +95,7 @@ TERM    term    = {
         0,
 	MARGIN,
 	SCRSIZ,
+	MROW,
         ttopen,
         ttclose,
         mswin_getc,
@@ -119,14 +109,31 @@ TERM    term    = {
 };
 
 
-
-int ttresize (row, col)
+/*
+ * mswin_resize - windows specific callback to set pico's internal tables
+ *		  to new screen dimensions.
+ */
+int
+mswin_resize (row, col)
+    int row, col;
 {
     resize_pico (row-1, col);
     return (0);
 }
 
 
+/*
+ * ttresize - recompute the screen dimensions if necessary, and then
+ *	      adjust pico's internal buffers accordingly.
+ */
+int ttresize ()
+{
+    int row, col;
+
+    mswin_getscreensize(&row, &col);
+    resize_pico (row-1, col);
+}
+
 
 /*
  * This function is called once to set up the terminal device streams.
@@ -149,7 +156,7 @@ ttopen()
     inschar  = delchar = FALSE;
     revexist = TRUE;
     
-    mswin_setresizecallback (ttresize);
+    mswin_setresizecallback (mswin_resize);
     
     
 
@@ -203,7 +210,7 @@ mouseoff()
 int
 ttclose()
 {
-    mswin_clearresizecallback (ttresize);
+    mswin_clearresizecallback (mswin_resize);
     return(1);
 }
 
@@ -233,26 +240,22 @@ GetKey ()
     int			ch = 0;
     unsigned int	lch;
     long		timein;
-    int			controlKey;
     MEvent		mouse;
     
-    
-    controlKey = FALSE;			/* xxx Don't know how to tell. */
-    ch = NODATA;
 
-    
+    ch = NODATA;
     timein = time(0L);
 
 
     /*
      * Main character processing loop.
      */
-    while((ch = mswin_getc ()) == MSWIN_KEY_NODATA) {
+    while(!mswin_charavail()) {
 
 #if MOUSE
 	/* Check Mouse.  If we get a mouse event, convert to char
 	 * event and return that. */
-	if (checkmouse (&ch)) {
+	if (checkmouse (&ch,0,0,0)) {
 	    curwp->w_flag |= WFHARD;
 	    return (ch);
 	}
@@ -265,7 +268,7 @@ GetKey ()
     }
 
     
-    return (MapMSKEYtoPK (ch));
+    return (MapMSKEYtoPK (mswin_getc_fast()));
 }
 
 
@@ -278,7 +281,9 @@ MapMSKEYtoPK (int c)
 	case MSWIN_KEY_DOWN:		return (K_PAD_DOWN);
 	case MSWIN_KEY_RIGHT:		return (K_PAD_RIGHT);
 	case MSWIN_KEY_LEFT:		return (K_PAD_LEFT);
+	case MSWIN_KEY_SCROLLUPPAGE:
 	case MSWIN_KEY_PREVPAGE:	return (K_PAD_PREVPAGE);
+	case MSWIN_KEY_SCROLLDOWNPAGE:
 	case MSWIN_KEY_NEXTPAGE:	return (K_PAD_NEXTPAGE);
 	case MSWIN_KEY_HOME:		return (K_PAD_HOME);
 	case MSWIN_KEY_END:		return (K_PAD_END);
@@ -295,6 +300,9 @@ MapMSKEYtoPK (int c)
 	case MSWIN_KEY_F10:		return (F10);
 	case MSWIN_KEY_F11:		return (F11);
 	case MSWIN_KEY_F12:		return (F12);
+	case MSWIN_KEY_SCROLLUPLINE:	return (K_SCROLLUPLINE);
+	case MSWIN_KEY_SCROLLDOWNLINE:	return (K_SCROLLDOWNLINE);
+	case MSWIN_KEY_SCROLLTO:	return (K_SCROLLTO);
 	case MSWIN_KEY_NODATA:		return (NODATA);
     }
     
@@ -315,93 +323,110 @@ MapMSKEYtoPK (int c)
  * checkmouse - Check mouse and return maped command.
  *
  *	EXPORTED to pico.
+ *      NOTE: "down", "xxx", and "yyy" aren't used under windows.
  */
 int	
-checkmouse (unsigned int *ch)
+checkmouse (unsigned int *ch, int ddd, int xxx, int yyy)
 {
     static int	oindex;		/* Index of previous mouse down. */
     int		k;		/* current bit/button of mouse */
     int		mcol;		/* current mouse column */
     int		mrow;		/* current mouse row */
     int		down;		/* TRUE when mouse down event. */
+    unsigned long r;
     int		rv = 0;		/* TRUE when we have something to return. */
     MEvent      mouse;
+    int		i = 0;
+    MENUITEM	*mp;
 
     
-    
     *ch = 0;
     
     /* Mouse installed? */
     if (!mexist)
 	return (FALSE);
 
-    if (!WinGetMEvent (&mouse)) 
+    if (!mswin_getmouseevent (&mouse)) 
 	return (FALSE);
 
 
     /* Location of mouse event. */
     mcol = mouse.nColumn;
     mrow = mouse.nRow;
-
-
-    /* Only pay attation to button 1. */
-    if (mouse.button == 1) {
-	int i = 0;
-
-	/* Mouse down or up? */
-	down = FALSE;
-	if (mouse.event == MEVENT_MOUSEDOWN) {	/* button down */
-	    down = TRUE;
-	    oindex = -1;	/* No Previous mouse down. */
-        }
-
-	
-	/* In special screen region? */
-	if(mfunc.action && M_ACTIVE(mrow, mcol, &mfunc)){
-	    unsigned long r;
-
-	    r = (*mfunc.action)(down, mrow, mcol);
-	    if (r & 0xffff){
-		*ch = (unsigned) ((r>>16) & 0xffff);
-		rv  = TRUE;
-	    }
+    
+    
+    
+    /* 
+     * If there is a tracking function it gets all the mouse events
+     * reguardless of where they occur.
+     */
+    if (mtrack != NULL) {
+	r = mtrack (mouse.event, mrow, mcol, mouse.button, mouse.keys);
+	if (r & 0xffff){
+	    *ch = (unsigned) ((r>>16) & 0xffff);
+	    rv  = TRUE;
 	}
-	else{
-		
-	    /* In any of the menuitems? */
-	    while(1){	/* see if we understand event */
-		if(i >= 12){
-		    i = -1;	/* Not Found. */
-		    break;
-		}
-
-		if(M_ACTIVE(mrow, mcol, &menuitems[i]))
-		    break;	/* Found. */
-
-		i++;		/* Next. */
-	    }
-
-	    /* Now, was that a mouse down or mouse up? */
-	    if (down) {				/* button down */
-		oindex = i;			/* remember where */
-		if(i != -1)			/* invert label */
-		    invert_label (1, &menuitems[i]);
-	    }
-	    else {				/* button up */
-		if (oindex != -1) {		/* If up in menu item. */
-		    if (i == oindex){		/* And same item down in. */
-			*ch = menuitems[i].val; /* Return menu character. */
-			rv = 1;
-		    }
-		}
-	    }
-	}
-
-	/* If this is mouse up AND there was a mouse down in a menu item
-	 * then uninvert that menu item */
-	if(!down && oindex != -1)
-	  invert_label(0, &menuitems[oindex]);	/* restore label */
+	return (rv);
     }
+    
+
+
+
+    /* Mouse down or up? */
+    if (mouse.event == M_EVENT_DOWN) {	/* button down */
+	oindex = -1;	/* No Previous mouse down. */
+    }
+
+
+    /* In special screen region? */
+    for(mp = mfunc; mp; mp = mp->next)
+      if(mp->action && M_ACTIVE(mrow, mcol, mp))
+	break;
+
+    if(mp){
+
+	r = (*mp->action)(mouse.event, mrow, mcol, mouse.button, mouse.keys);
+	if (r & 0xffff){
+	    *ch = (unsigned) ((r>>16) & 0xffff);
+	    rv  = TRUE;
+	}
+    }
+    else{
+
+	/* In any of the menuitems? */
+	while(1){	/* see if we understand event */
+	    if(i >= 12){
+		i = -1;	/* Not Found. */
+		break;
+	    }
+
+	    if(M_ACTIVE(mrow, mcol, &menuitems[i]))
+		break;	/* Found. */
+
+	    i++;		/* Next. */
+	}
+
+	/* Now, was that a mouse down or mouse up? */
+	if (mouse.event == M_EVENT_DOWN) {	/* button down */
+	    oindex = i;			/* remember where */
+	    if(i != -1)			/* invert label */
+		invert_label (1, &menuitems[i]);
+	}
+	else if (mouse.event == M_EVENT_UP) {/* button up */
+	    if (oindex != -1) {		  /* If up in menu item. */
+		if (i == oindex){	  /* And same item down in. */
+		    *ch = menuitems[i].val; /* Return menu character. */
+		    rv = 1;
+		}
+	    }
+	}
+    }
+
+    /* If this is mouse up AND there was a mouse down in a menu item
+     * then uninvert that menu item */
+    if(mouse.event == M_EVENT_UP && oindex != -1)
+      invert_label(0, &menuitems[oindex]);	/* restore label */
+
     return(rv);
 }
 
@@ -409,7 +434,7 @@ checkmouse (unsigned int *ch)
 /*
  * invert_label - highlight the label of the given menu item.
  */
-static void
+void
 invert_label(state, m)
 int state;
 MENUITEM *m;
@@ -417,22 +442,27 @@ MENUITEM *m;
     int			i, j, r, c, p;
     char		*lp;
     int			old_state;
-	int			wasShown;
+    int			wasShown;
+    int			col_offset;
 
     if(m->val == mnoop)
       return;
 
   
-	mswin_getpos (&r, &c);			/* get cursor position */
-	wasShown = mswin_showcursor (0);
+    mswin_getpos (&r, &c);			/* get cursor position */
+    wasShown = mswin_showcursor (0);
     old_state = mswin_getrevstate ();
-    (*term.t_move)(m->tl.r, m->tl.c);
+    /*
+     * Leave the command name bold
+     */
+    col_offset = (state || !(lp=strchr(m->label, ' '))) ? 0 : (lp - m->label);
+    (*term.t_move)(m->tl.r, m->tl.c + col_offset);
     (*term.t_rev)(state);
 
-	for(i = m->tl.r; i <= m->br.r; i++) {
-      for(j = m->tl.c; j <= m->br.c; j++) {
-        if(i == m->lbl.r && j == m->lbl.c){	/* show label?? */
-	      lp = m->label;
+    for(i = m->tl.r; i <= m->br.r; i++) {
+      for(j = m->tl.c + col_offset; j <= m->br.c; j++) {
+        if(i == m->lbl.r && j == m->lbl.c + col_offset){ /* show label?? */
+	      lp = m->label + col_offset;
 	      while(*lp && j++ < m->br.c)
 	        (*term.t_putchar)(*lp++);
 
@@ -451,6 +481,114 @@ MENUITEM *m;
 
 
 
+/*
+ * Called by mswin to scroll text in window in responce to the scrollbar.
+ *
+ *  Args: cmd - what type of scroll operation.
+ * 	scroll_pos - paramter for operation.  
+ *			used as position for SCROLL_TO operation.
+ * 
+ *  Returns: TRUE - did the scroll operation.
+ *	   FALSE - was not able to do the scroll operation.
+ */
+int
+pico_scroll_callback (cmd, scroll_pos)
+int	cmd;
+long	scroll_pos;
+{
+    switch (cmd) {
+    case MSWIN_KEY_SCROLLUPLINE:
+	scrollupline (0, 1);
+	break;
+
+    case MSWIN_KEY_SCROLLDOWNLINE:
+        scrolldownline (0, 1);
+	break;
+		
+    case MSWIN_KEY_SCROLLUPPAGE:
+	backpage (0, 1);
+	break;
+	
+    case MSWIN_KEY_SCROLLDOWNPAGE:
+	forwpage (0, 1);
+	break;
+	    
+    case MSWIN_KEY_SCROLLTO:
+	scrollto (0, 0);
+	break;
+    }
+    
+    update ();
+    return (TRUE);
+}
+
+
+
+/*
+ * Update the scroll range and position. (exported)
+ *
+ * This is where curbp->b_linecnt is really managed.  With out this function
+ * to count the number of lines when needed curbp->b_linecnt will never
+ * really be correct.  BUT, this function is only compiled into the 
+ * windows version, so b_linecnt will only ever be right in the windows
+ * version.  OK for now because that is the only version that
+ * looks at b_linecnt.
+ */
+update_scroll ()
+{
+    long	scr_pos;
+    long	scr_range;
+    LINE	*lp;
+    static LINE *last_top_line = NULL;
+    static long last_scroll_pos = -1;
+    
+    
+    if (ComposerEditing) {
+	/* Editing header - don't allow scroll bars. */
+	mswin_setscrollrange (0);
+	return(0);
+    }
+	   
+	
+    /*
+     * Count the number of lines in the current bufer.  Done when:
+     *
+     *      when told to recount:           curbp->b_linecnt == -1
+     *      when the top line changed:      curwp->w_linep != last_top_line
+     *  when we don't know the scroll pos:  last_scroll_pos == -1
+     *
+     * The first line in the list is a "place holder" line and is not
+     * counted.  The list is circular, when we return the to place
+     * holder we have reached the end.
+     */
+    if(curbp->b_linecnt == -1 || curwp->w_linep != last_top_line
+       || last_scroll_pos == -1) {
+	scr_range = 0;
+	scr_pos = 0;
+	for (lp = lforw (curbp->b_linep); lp != curbp->b_linep; 
+	     lp = lforw (lp)) {
+	    if (lp == curwp->w_linep)
+              scr_pos = scr_range;
+
+	    ++scr_range;
+	}
+
+	curbp->b_linecnt = scr_range;
+	last_scroll_pos = scr_pos;
+	last_top_line = curwp->w_linep;
+    }
+
+    /*
+     * Set new scroll range and position.
+     */
+    mswin_setscrollrange (curbp->b_linecnt - 1);
+    mswin_setscrollpos (last_scroll_pos);
+    return (0);
+}
+
+
+
+
 /*
  * alt_editor - fork off an alternate editor for mail message composition
  *
@@ -460,6 +598,7 @@ alt_editor(int f, int n)
 {
     char   eb[NLINE];				/* buf holding edit command */
     char   *fn;					/* tmp holder for file name */
+    char    errbuf[128];
     char   *writetmp();
     int	   status;
     int	   done;
@@ -560,28 +699,54 @@ alt_editor(int f, int n)
 	emlwrite("Alternate editor abandoned.", NULL);
 	return (-1);
 
-	
-	/*
-	 * Positive error codes are from WinExec.  The _really_ common
-	 * ones get interpreted.
-	 */
-    case 2:
-    case 3:
-	emlwrite("Alternate editor %s not found.", eb);
-	return (-1);
-	
-    case 8:
-	emlwrite("Not enough memory to run alternate editor.", NULL);
-	return (-1);
-	
     default:
-	emlwrite("Error %d starting alternate editor.",(void *)status);
+	mswin_exec_err_msg ("alternate editor", status, errbuf, 128);
+	emlwrite (errbuf, NULL);
 	return (-1);
     }
     return (-1);
 }
 
 
+/*
+ *
+ */
+void
+pico_config_menu_items (KEYMENU *keymenu)
+{
+    int		i;
+    KEYMENU	*k;
+    int		key;
+    char	cleanLabel[64];
+    char	*rb;
+
+    mswin_menuitemclear ();
+
+    /* keymenu's seem to be hardcoded at 12 entries. */
+    for (i = 0, k = keymenu; i < 12; ++i, ++k) {
+	if (k->name != NULL && k->label != NULL && 
+		k->menuitem != KS_NONE) {
+
+	    if (k->name[0] == '^')
+		key = CTRL | k->name[1];
+	    else if (strcmp(k->name, "Ret") == 0) 
+		key = '\r';
+	    else
+		key = k->name[0];
+
+	    key |= MENU;	/* Flag it as comming from a menu selection */
+
+	    if (k->label[0] == '[' && strchr (k->label, ']') != NULL) {
+	        strcpy (cleanLabel, &k->label[1]);
+		rb = strchr (cleanLabel, ']');
+		*rb = '\0';
+		mswin_menuitemadd (key, cleanLabel, k->menuitem, 0);
+	    }
+	    else
+	    	mswin_menuitemadd (key, k->label, k->menuitem, 0);
+	}
+    }
+}
 
 
 /*
@@ -626,6 +791,20 @@ FILE *fp;
 
 
 
+/*
+ * pico_file_browse - Exported version of FileBrowse below.
+ */
+pico_file_browse(pdata, dir, fn, sz, flags)
+PICO *pdata;
+char *dir, *fn, *sz;
+int   flags;
+{
+    return(FileBrowse(dir, fn, sz, flags));
+}
+
+
+
+
 
 
 
