@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: region.c,v 4.18 1996/03/15 07:41:11 hubert Exp $";
+static char rcsid[] = "$Id: region.c,v 4.19 1996/07/09 23:55:54 mikes Exp $";
 #endif
 /*
  * Program:	Region management routines
@@ -93,6 +93,41 @@ killregion(f, n)
 }
 
 
+/*
+ * Blast the region without saving . Ask "getregion"
+ * to figure out the bounds of the region.
+ * Move "." to the start, and kill the characters.
+ * Bound to "C-W".
+ */
+deleteregion(f, n)
+{
+    REGION          region;
+
+    if (curbp->b_mode&MDVIEW)		/* don't allow this command if	*/
+      return(rdonly());			/* we are in read only mode	*/
+
+    if (getregion(&region) == TRUE){
+	curwp->w_dotp = region.r_linep;
+	curwp->w_doto = region.r_offset;
+	curwp->w_markp = NULL;
+#ifdef	_WINDOWS
+	mswin_allowcopycut(NULL);
+#endif
+	if(ldelete(region.r_size, FALSE)){
+	    if(curwp->w_dotp == curwp->w_linep
+	       && curwp->w_dotp == curbp->b_linep){
+		curwp->w_force = 0;		/* Center dot. */
+		curwp->w_flag |= WFFORCE;
+	    }
+
+	    return(TRUE);
+	}
+    }
+
+    return (FALSE);
+}
+
+
 /*
  * Copy all of the characters in the
  * region to the kill buffer. Don't move dot
