@@ -20,11 +20,20 @@
  *		clipboard.  Fixed to limit amount of pasted text during
  *		text input prompts.  Windows printing added.
  *
+ * ....
+ *
+ * 3.92, Jan 22 1995
+ *		Recent modifications:  Menues, mouse sensitivity in file browser
+ *		(which isn't actually seen in the windows version), and responce
+ *		to MW_QUERYENDSESSION AND MW_ENDSESSION.
+ *		Saved source for this version for Dr. Watson logs for tracking
+ *		random protection faults.
+ *
  */
 
 
 #define VER_MAJOR 3
-#define VER_MINOR 91
+#define VER_MINOR 95
 extern char datestamp[];
 
 
